@@ -1,17 +1,22 @@
-%% This library is free software; you can redistribute it and/or modify
-%% it under the terms of the GNU Lesser General Public License as
-%% published by the Free Software Foundation; either version 2 of the
-%% License, or (at your option) any later version.
+%% Licensed under the Apache License, Version 2.0 (the "License"); you may
+%% not use this file except in compliance with the License. You may obtain
+%% a copy of the License at <http://www.apache.org/licenses/LICENSE-2.0>
 %%
-%% This library is distributed in the hope that it will be useful, but
-%% WITHOUT ANY WARRANTY; without even the implied warranty of
-%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-%% Lesser General Public License for more details.
+%% Unless required by applicable law or agreed to in writing, software
+%% distributed under the License is distributed on an "AS IS" BASIS,
+%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+%% See the License for the specific language governing permissions and
+%% limitations under the License.
 %%
-%% You should have received a copy of the GNU Lesser General Public
-%% License along with this library; if not, write to the Free Software
-%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-%% USA
+%% Alternatively, you may use this file under the terms of the GNU Lesser
+%% General Public License (the "LGPL") as published by the Free Software
+%% Foundation; either version 2.1, or (at your option) any later version.
+%% If you wish to allow use of your version of this file only under the
+%% terms of the LGPL, you should delete the provisions above and replace
+%% them with the notice and other provisions required by the LGPL; see
+%% <http://www.gnu.org/licenses/>. If you do not delete the provisions
+%% above, a recipient may use your version of this file under the terms of
+%% either the Apache License or the LGPL.
 %%
 %% @copyright 2004-2007 Mickaël Rémond, Richard Carlsson
 %% @author Mickaël Rémond <mickael.remond@process-one.net>
@@ -102,7 +107,7 @@ format_stacktrace(Trace) ->
     format_stacktrace(Trace, "in function", "in call from").
 
 format_stacktrace([{M,F,A,L}|Fs], Pre, Pre1) when is_integer(A) ->
-    [io_lib:fwrite("~ts ~w:~w/~w~ts\n",
+    [io_lib:fwrite("~ts ~w:~tw/~w~ts\n",
                    [Pre, M, F, A, format_stacktrace_location(L)])
      | format_stacktrace(Fs, Pre1, Pre1)];
 format_stacktrace([{M,F,As,L}|Fs], Pre, Pre1) when is_list(As) ->
@@ -116,9 +121,9 @@ format_stacktrace([{M,F,As,L}|Fs], Pre, Pre1) when is_list(As) ->
 		io_lib:fwrite("~ts ~ts ~ts",
 			      [format_arg(A1),F,format_arg(A2)]);
 	    false ->
-		io_lib:fwrite("~w(~ts)", [F,format_arglist(As)])
+		io_lib:fwrite("~tw(~ts)", [F,format_arglist(As)])
 	end,
-    [io_lib:fwrite("~ts ~w:~w/~w~ts\n  called as ~ts\n",
+    [io_lib:fwrite("~ts ~w:~tw/~w~ts\n  called as ~ts\n",
 		   [Pre,M,F,A,format_stacktrace_location(L),C])
      | format_stacktrace(Fs,Pre1,Pre1)];
 format_stacktrace([{M,F,As}|Fs], Pre, Pre1) ->
@@ -157,15 +162,15 @@ is_op(_M, _F, _A) ->
 format_error({bad_test, Term}) ->
     error_msg("bad test descriptor", "~tP", [Term, 15]);
 format_error({bad_generator, {{M,F,A}, Term}}) ->
-    error_msg(io_lib:format("result from generator ~w:~w/~w is not a test",
+    error_msg(io_lib:format("result from generator ~w:~tw/~w is not a test",
                             [M,F,A]),
               "~tP", [Term, 15]);
 format_error({generator_failed, {{M,F,A}, Exception}}) ->
-    error_msg(io_lib:format("test generator ~w:~w/~w failed",[M,F,A]),
+    error_msg(io_lib:format("test generator ~w:~tw/~w failed",[M,F,A]),
               "~ts", [format_exception(Exception)]);
 format_error({no_such_function, {M,F,A}})
   when is_atom(M), is_atom(F), is_integer(A) ->
-    error_msg(io_lib:format("no such function: ~w:~w/~w", [M,F,A]),
+    error_msg(io_lib:format("no such function: ~w:~tw/~w", [M,F,A]),
 	      "", []);
 format_error({module_not_found, M}) ->
     error_msg("test module not found", "~tp", [M]);
@@ -180,7 +185,7 @@ format_error({cleanup_failed, Exception}) ->
     error_msg("context cleanup failed", "~ts",
 	      [format_exception(Exception)]);
 format_error({{bad_instantiator, {{M,F,A}, Term}}, _DummyException}) ->
-    error_msg(io_lib:format("result from instantiator ~w:~w/~w is not a test",
+    error_msg(io_lib:format("result from instantiator ~w:~tw/~w is not a test",
                             [M,F,A]),
               "~tP", [Term, 15]);
 format_error({instantiation_failed, Exception}) ->
@@ -379,11 +384,9 @@ fun_parent(F) ->
 	    {arity, A} = erlang:fun_info(F, arity),
 	    {M, N, A};
 	{type, local} ->
-	    [$-|S] = atom_to_list(N),
-	    C1 = string:chr(S, $/),
-	    C2 = string:chr(S, $-),
-	    {M, list_to_atom(string:sub_string(S, 1, C1 - 1)),
-	     list_to_integer(string:sub_string(S, C1 + 1, C2 - 1))}
+            [$-|S] = atom_to_list(N),
+            [S2, T] = string:split(S, "/", trailing),
+            {M, list_to_atom(S2), element(1, string:to_integer(T))}
     end.
 
 -ifdef(TEST).
@@ -399,6 +402,7 @@ uniq([X | Xs]) -> [X | uniq(Xs)];
 uniq([]) -> [].
 
 -ifdef(TEST).
+-dialyzer({[no_fail_call, no_improper_lists], uniq_test_/0}).
 uniq_test_() ->
     {"uniq",
      [?_assertError(function_clause, uniq(ok)),
@@ -459,6 +463,7 @@ normalize([]) ->
 
 -ifdef(TEST).
 
+-dialyzer({no_match, cmd_test_/0}).
 cmd_test_() ->
     ([?_test({0, "hello\n"} = ?_cmd_("echo hello"))]
      ++ case os:type() of
